-module(trick1a).
-author("Doug Edmunds").
% dougedmunds AT gmail DOT com
% based on approach to Cheney card trick 
% in "n-Card Tricks" by Hang Chen and Curtis Cooper
% The College Mathematics Journal
% Vol. 40, No. 3 (May 2009), pp. 196-201
% free view at http://www.jstor.org/stable/25653718
% "someone in the audience picks five cards, then selects one as the hidden card"  

-compile(export_all).
-include_lib("wx/include/wx.hrl").

start() ->
    State = make_window(),
    loop (State), 
    wx:destroy().

make_window() ->
    ETS = make_ets(),
    Server = wx:new(),
    Frame = wxFrame:new(Server, -1, "The Secret Card (by Doug Edmunds)", [{size,{650, 465}}]),
    Panel  = wxPanel:new(Frame),
	SelectedCards = [], %%cards selected
    HiddenCard = 0, %%card selected from the 5 as hidden
	ButtonFlag = true,
	
%% create widgets
%% the order entered here does not control appearance
    ST2001 = wxStaticText:new(Panel, 2001,"Select exactly 5 cards, then press the button below",[]),
    TC3001 = wxTextCtrl:new(Panel, 3001,[{value,""}, {size, {200,25}}]),
	wxTextCtrl:setEditable(TC3001, false),
    B101  = wxButton:new(Panel, 101, [{label, "&Make 5 selections then press here"}]),
    B102  = wxButton:new(Panel, ?wxID_EXIT, [{label, "E&xit"}]),
	

%%You can create sizers before or after the widgets that will go into them, but
%%the widgets have to exist before they are added to sizer.
    OuterSizer  = wxBoxSizer:new(?wxHORIZONTAL),
    MainSizer   = wxBoxSizer:new(?wxVERTICAL),
    ButtonSizer = wxBoxSizer:new(?wxVERTICAL),

%% Note that the widget is added using the VARIABLE, not the ID.
%% The order they are added here controls appearance.

    CheckBoxes_S = create_checkboxes_spades(Panel),
    CheckBoxes_H = create_checkboxes_hearts(Panel),
    CheckBoxes_D = create_checkboxes_diamonds(Panel),
    CheckBoxes_C = create_checkboxes_clubs(Panel),

    wxSizer:addSpacer(MainSizer, 20),
    wxSizer:add(MainSizer, CheckBoxes_S),
    wxSizer:addSpacer(MainSizer, 20),
    wxSizer:add(MainSizer, CheckBoxes_H),
    wxSizer:addSpacer(MainSizer, 20),
    wxSizer:add(MainSizer, CheckBoxes_D),
    wxSizer:addSpacer(MainSizer, 20),
    wxSizer:add(MainSizer, CheckBoxes_C),
    wxSizer:addSpacer(MainSizer, 20),
     	

    wxSizer:addSpacer(MainSizer, 10),  %spacer
    wxSizer:add(MainSizer, ST2001, []),
    wxSizer:addSpacer(MainSizer, 40),  %spacer
    wxSizer:add(MainSizer, TC3001, []),
    wxSizer:addSpacer(MainSizer, 10),  %spacer

    wxSizer:add(ButtonSizer, B101,  []),
	wxSizer:addSpacer(ButtonSizer, 10),  %spacer
    wxSizer:add(ButtonSizer, B102,  []),  %exit button
    wxSizer:add(MainSizer, ButtonSizer, []),

    wxSizer:addSpacer(OuterSizer, 20), % spacer
    wxSizer:add(OuterSizer, MainSizer, []),

%% Now 'set' OuterSizer into the Panel
    wxPanel:setSizer(Panel, OuterSizer),

    wxFrame:show(Frame),

    % create listeners for closing alternatives
    wxFrame:connect( Frame, close_window),
    wxPanel:connect(Panel, command_button_clicked),
	
    %% the return value, which is stored in State
    {Frame, Panel, ST2001, TC3001, SelectedCards, HiddenCard, ETS, ButtonFlag}.  %%State

create_checkboxes_spades(Panel) ->
    CheckSizer = wxStaticBoxSizer:new(?wxHORIZONTAL, Panel, 
				      [{label, "Spades"}]),
    
    CheckBoxes =
	[wxCheckBox:new(Panel, 6401, "A  ", []),
	 wxCheckBox:new(Panel, 6402, "2  ", []),
	 wxCheckBox:new(Panel, 6403, "3  ", []),
	 wxCheckBox:new(Panel, 6404, "4  ", []),
	 wxCheckBox:new(Panel, 6405, "5  ", []),
	 wxCheckBox:new(Panel, 6406, "6  ", []),
	 wxCheckBox:new(Panel, 6407, "7  ", []),
	 wxCheckBox:new(Panel, 6408, "8  ", []),
	 wxCheckBox:new(Panel, 6409, "9  ", []),
	 wxCheckBox:new(Panel, 6410, "T  ", []),
	 wxCheckBox:new(Panel, 6411, "J  ", []),
	 wxCheckBox:new(Panel, 6412, "Q  ", []),
	 wxCheckBox:new(Panel, 6413, "K  ", [])
	],
    Fun =
	fun(Item) ->
		wxCheckBox:connect(Item, command_checkbox_clicked),
		wxSizer:add(CheckSizer, Item)
	end,
    wx:foreach(Fun, CheckBoxes),
    CheckSizer.

create_checkboxes_hearts(Panel) ->
    CheckSizer = wxStaticBoxSizer:new(?wxHORIZONTAL, Panel, 
				      [{label, "Hearts"}]),
    
    CheckBoxes =
	[wxCheckBox:new(Panel, 6301, "A  ", []),
	 wxCheckBox:new(Panel, 6302, "2  ", []),
	 wxCheckBox:new(Panel, 6303, "3  ", []),
	 wxCheckBox:new(Panel, 6304, "4  ", []),
	 wxCheckBox:new(Panel, 6305, "5  ", []),
	 wxCheckBox:new(Panel, 6306, "6  ", []),
	 wxCheckBox:new(Panel, 6307, "7  ", []),
	 wxCheckBox:new(Panel, 6308, "8  ", []),
	 wxCheckBox:new(Panel, 6309, "9  ", []),
	 wxCheckBox:new(Panel, 6310, "T  ", []),
	 wxCheckBox:new(Panel, 6311, "J  ", []),
	 wxCheckBox:new(Panel, 6312, "Q  ", []),
	 wxCheckBox:new(Panel, 6313, "K  ", [])
	],
    Fun =
	fun(Item) ->
		wxCheckBox:connect(Item, command_checkbox_clicked),
		wxSizer:add(CheckSizer, Item)
	end,
    wx:foreach(Fun, CheckBoxes),
    CheckSizer.

create_checkboxes_diamonds(Panel) ->
    CheckSizer = wxStaticBoxSizer:new(?wxHORIZONTAL, Panel, 
				      [{label, "Diamonds"}]),
    
    CheckBoxes =
	[wxCheckBox:new(Panel, 6201, "A  ", []),
	 wxCheckBox:new(Panel, 6202, "2  ", []),
	 wxCheckBox:new(Panel, 6203, "3  ", []),
	 wxCheckBox:new(Panel, 6204, "4  ", []),
	 wxCheckBox:new(Panel, 6205, "5  ", []),
	 wxCheckBox:new(Panel, 6206, "6  ", []),
	 wxCheckBox:new(Panel, 6207, "7  ", []),
	 wxCheckBox:new(Panel, 6208, "8  ", []),
	 wxCheckBox:new(Panel, 6209, "9  ", []),
	 wxCheckBox:new(Panel, 6210, "T  ", []),
	 wxCheckBox:new(Panel, 6211, "J  ", []),
	 wxCheckBox:new(Panel, 6212, "Q  ", []),
	 wxCheckBox:new(Panel, 6213, "K  ", [])
	],
    Fun =
	fun(Item) ->
		wxCheckBox:connect(Item, command_checkbox_clicked),
		wxSizer:add(CheckSizer, Item)
	end,
    wx:foreach(Fun, CheckBoxes),
    CheckSizer.
		
create_checkboxes_clubs(Panel) ->
    CheckSizer = wxStaticBoxSizer:new(?wxHORIZONTAL, Panel, 
				      [{label, "Clubs"}]),
    
    CheckBoxes =
	[wxCheckBox:new(Panel, 6101, "A  ", []),
	 wxCheckBox:new(Panel, 6102, "2  ", []),
	 wxCheckBox:new(Panel, 6103, "3  ", []),
	 wxCheckBox:new(Panel, 6104, "4  ", []),
	 wxCheckBox:new(Panel, 6105, "5  ", []),
	 wxCheckBox:new(Panel, 6106, "6  ", []),
	 wxCheckBox:new(Panel, 6107, "7  ", []),
	 wxCheckBox:new(Panel, 6108, "8  ", []),
	 wxCheckBox:new(Panel, 6109, "9  ", []),
	 wxCheckBox:new(Panel, 6110, "T  ", []),
	 wxCheckBox:new(Panel, 6111, "J  ", []),
	 wxCheckBox:new(Panel, 6112, "Q  ", []),
	 wxCheckBox:new(Panel, 6113, "K  ", [])
	],
    Fun =
	fun(Item) ->
		wxCheckBox:connect(Item, command_checkbox_clicked),
		wxSizer:add(CheckSizer, Item)
	end,
    wx:foreach(Fun, CheckBoxes),
    CheckSizer.

%%%%%%%%%%%%% Calculation Section %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% sample{6304,15,3,4,"4h"}	 {id, OverallRank, SuitNumber, FaceNumber, Label}

first_card(SelectedCards,HiddenCard) ->
    %suit of the hidden card
	%rank visible cards, lowest for clubs, highest for spades
	%sort visible by rank (1-52)
	VisibleCards = lists:delete(HiddenCard, SelectedCards),
%	io:format("VisibleCards: ~p~n",[VisibleCards]),
	VisibleCardsRanked = lists:sort(lists:flatten([ets:match(ets_table,{C,'$1','_','_','_'}) || C <-VisibleCards])),
%	io:format("VisibleCardsRanked: ~p~n",[VisibleCardsRanked]),
	[HiddenCardSuitNumber] = lists:flatten(ets:match(ets_table,{HiddenCard,'_','$1','_','_'})),
%	io:format("HiddenCardSuitNumber: ~p~n",[HiddenCardSuitNumber]),
    %first card of 4 indicates suit (lowest is C, then DHS)
    FirstCardRank = lists:nth(HiddenCardSuitNumber,VisibleCardsRanked),
%	io:format("FirstCardRank: ~p~n",[FirstCardRank]),
	_FirstCardLabel =  lists:flatten(ets:match(ets_table,{'_',FirstCardRank,'_','_','$1'})),
%	io:format("FirstCardLabel: ~p~n",[_FirstCardLabel]),
	[FirstCard] = lists:flatten(ets:match(ets_table,{'$1',FirstCardRank,'_','_','_'})),
%	io:format("FirstCard: ~p~n",[FirstCard]),
	FirstCard.
	
second_card(SelectedCards,HiddenCard,FirstCard) ->
    %is face (1-13) of HiddenCard higher or lower than FirstCard
	%distance uses mod13 concept, max gap is 6 one way, 7 the other, making the 6 way closer. 
	%send the distance info out, so as not to have to recalculate it for last 3
	[HiddenCardFaceNumber] = lists:flatten(ets:match(ets_table,{HiddenCard,'_','_','$1','_'})),
%	io:format("HiddenCardFaceNumber: ~p~n",[HiddenCardFaceNumber]),	
	[FirstCardFaceNumber] = lists:flatten(ets:match(ets_table,{FirstCard,'_','_','$1','_'})),
%	io:format("FirstCardFaceNumber: ~p~n",[FirstCardFaceNumber]),	

	H = HiddenCardFaceNumber, F = FirstCardFaceNumber,
	
	%if H < F, add 13 to H.  Calc H - F, if result is <7, then select higher, otherwise select lower 
	if H < F -> Clockwise = H + 13 - F;
		true -> Clockwise = H - F
	end,

	if F < H -> CounterClockwise = F + 13 - H;
		true -> CounterClockwise = F - H 
	end,

%	io:format("Clockwise: ~p~n",[Clockwise]),
%	io:format("CounterClockwise: ~p~n",[CounterClockwise]),

	
    if 
		Clockwise < CounterClockwise  -> SecondCardPick = 3, 
			Diff = Clockwise; %closest going clockwise, means go up, add to first
		Clockwise == CounterClockwise -> SecondCardPick = 2,
			Diff = 0; %same rank, other 3 cards are irrelevant
        true           				  -> SecondCardPick = 1, % Clockwise > CounterClockwise
			Diff = CounterClockwise %closest going counterclockwise, means go down, subtract from first
	end,
	
	
	%get remaining 3 cards
	VisibleCards1 = lists:delete(HiddenCard, SelectedCards),
	VisibleCards2 = lists:delete(FirstCard, VisibleCards1),
%	io:format("VisibleCards2: ~p~n",[VisibleCards2]),
	VisibleCards2Rank = lists:sort(lists:flatten([ets:match(ets_table,{C,'$1','_','_','_'}) || C <-VisibleCards2])),
%	io:format("VisibleCards2Rank: ~p~n",[VisibleCards2Rank]),
    SecondCardRank = lists:nth(SecondCardPick, VisibleCards2Rank),
%	io:format("SecondCardRank: ~p~n",[SecondCardRank]),
	_SecondCardLabel =  lists:flatten(ets:match(ets_table,{'_',SecondCardRank,'_','_','$1'})),
%	io:format("SecondCardLabel: ~p~n",[SecondCardLabel]),
	[SecondCard] = lists:flatten(ets:match(ets_table,{'$1',SecondCardRank,'_','_','_'})),
%	io:format("SecondCard: ~p~n",[SecondCard]),
	{SecondCard, Diff}.

last_three_cards(SelectedCards,HiddenCard,FirstCard,SecondCard, Diff) ->
	%get remaining 2 cards
	VisibleCards1 = lists:delete(HiddenCard, SelectedCards),
	VisibleCards2 = lists:delete(FirstCard, VisibleCards1),
	VisibleCards3 = lists:delete(SecondCard, VisibleCards2),
%	io:format("VisibleCards3: ~p~n ",[VisibleCards3]),
	[LRank, URank] = _VisibleCards3Rank = lists:sort(lists:flatten([ets:match(ets_table,{C,'$1','_','_','_'}) || C <-VisibleCards3])),
%	io:format("VisibleCards3Rank: ~p~n",[VisibleCards3Rank]),
	[L] = lists:flatten(ets:match(ets_table,{'$1',LRank,'_','_','_'})),
%	io:format("L: ~p~n",[L]),
	[U] = lists:flatten(ets:match(ets_table,{'$1',URank,'_','_','_'})),
%	io:format("U: ~p~n",[U]),
	
%	io:format("Diff: ~p~n ",[Diff]),
	%represent Diff (1-6) thusly, LowerVisible, UpperVisible, HiddenCard
  %H 1st: 1 or 2; H 2nd: 3 or 4; H 3rd: 5 or 6,  
	%[L,U,H] = 1,  [L,H,U] = 2, [U,L,H] = 3,  
	%[U,H,L] = 4,  [H,L,U] = 5, [H,U,L] = 6,
  % 0 doesn't matter  
	case Diff of
		1 -> LastThreeCards = [L,U,HiddenCard];
		2 -> LastThreeCards = [L,HiddenCard,U];
		3 -> LastThreeCards = [U,L,HiddenCard];
		4 -> LastThreeCards = [U,HiddenCard,L];
		5 -> LastThreeCards = [HiddenCard,L,U];
		6 -> LastThreeCards = [HiddenCard,U,L];
		0 -> LastThreeCards = [HiddenCard,L,U];  %doesn't matter
		_ -> io:format("ERROR ~n ",[]), LastThreeCards = [HiddenCard,L,U]
	end,	
	LastThreeCards.
	
calculate_layout(SelectedCards,HiddenCard) ->
%	io:format("SelectedCards: ~p~n", [SelectedCards]),
%	io:format("HiddenCard: ~p~n", [HiddenCard]),
	case length(SelectedCards) of
	   5 -> FirstCard = first_card(SelectedCards,HiddenCard),
			{SecondCard,Diff} =second_card(SelectedCards,HiddenCard,FirstCard),
			FirstCardLabel = lists:flatten(ets:match(ets_table,{FirstCard,'_','_','_','$1'})),
			SecondCardLabel = lists:flatten(ets:match(ets_table,{SecondCard,'_','_','_','$1'})),
			[L1, L2, L3] = _LastThreeCards = last_three_cards(SelectedCards,HiddenCard,FirstCard,SecondCard, Diff),
%			io:format("LastThreeCards: ~p~n", [LastThreeCards]),
			
			case L1 of 
				HiddenCard -> L1Label = "XX";
				_ -> L1Label = lists:flatten(ets:match(ets_table,{L1,'_','_','_','$1'}))
			end,
			
			case L2 of 
				HiddenCard -> L2Label = "XX";
				_ -> L2Label = lists:flatten(ets:match(ets_table,{L2,'_','_','_','$1'}))
			end,

			case L3 of 
				HiddenCard -> L3Label = "XX";
				_ -> L3Label = lists:flatten(ets:match(ets_table,{L3,'_','_','_','$1'}))
			end,

			Output = FirstCardLabel ++ " " ++ SecondCardLabel ++ " " ++ L1Label ++ " " ++ L2Label ++ " " ++ L3Label;
	   _ -> Output = "Error in card count"
	end,   
	Output.
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
loop(State) ->
    {Frame, Panel, ST2001, TC3001,  SelectedCards, HiddenCard, ETS, ButtonFlag}  = State,  % break State back down into its components
%    io:format("--waiting in the loop--~n", []), % optional, feedback to the shell
    receive

        % a connection gets the close_window signal
        % and sends this message to the server
        #wx{event=#wxClose{}} ->
 %           io:format("~p Closing window ~n",[self()]), %optional, goes to shell
            %now we use the reference to Frame
            wxWindow:destroy(Frame),  %closes the window
			ets:delete(ets_table),
            ok;  % we exit the loop

        #wx{id = ?wxID_EXIT, event=#wxCommand{type = command_button_clicked} } ->
            %this message is sent when the exit button (ID 102) is clicked
            %the other fields in the tuple are not important to us.
 %           io:format("~p Closing window ~n",[self()]), %optional, goes to shell
            wxWindow:destroy(Frame),
			ets:delete(ets_table),
            ok;  % we exit the loop

		#wx{id = 101, event=#wxCommand{type = command_button_clicked} } ->
            %the selected 5 button (ID 101) is clicked
 %           io:format("Selected 5 Button clicked ~n",[]), 
			case ButtonFlag of 
			true ->
				case length(SelectedCards) of
					5 ->
						Choices = [ets:match(ets_table,{C,'_','_','_','$1'}) ++ " " || C <- SelectedCards],
						RadioBox2 = wxRadioBox:new(Panel, 7771, "Select your secret card.",
							{350, 300},
							?wxDefaultSize,
							Choices,
							[{majorDim, 3}, {style, ?wxHORIZONTAL}]),
						wxRadioBox:connect(RadioBox2, command_radiobox_selected),

						%enable button to process based on hidden card
						%SelectedCards maintains order of picking (bumping up, if lower unchecked)
                        %radio buttons default to firstmost SelectCards, so pick that one
                        NewHiddenCard = lists:nth(1,SelectedCards),
%						io:format("NewHiddenCard: ~p~n", [NewHiddenCard]),
						wxButton:new(Panel, 103, [{label, "&Pick one, click here to finish"}, {pos,{350,375}}]);

					_ ->  wxStaticText:setLabel(ST2001, "If too many, uncheck some"),
%					    io:format("Wrong count, must be 5 ~n", []),
						NewHiddenCard = HiddenCard,
						true
				end;
			false -> NewHiddenCard = HiddenCard
			end,			
			NewState = {Frame, Panel, ST2001, TC3001, SelectedCards, NewHiddenCard, ETS, ButtonFlag},
			loop(NewState);		

		#wx{id = 103, event=#wxCommand{type = command_button_clicked} } ->
%            io:format("Hidden Selection Done button clicked ~n",[]), 
			case ButtonFlag of
			true ->
				Output = calculate_layout(SelectedCards,HiddenCard),
				HiddenCardLabel = lists:flatten(ets:match(ets_table,{HiddenCard,'_','_','_','$1'})),

				wxStaticText:setLabel(ST2001, "Your secret card is the " ++ HiddenCardLabel ++ ". Remember it!\nCopy the line below and give it to me.\nI will tell you your secret card." ),			
				wxTextCtrl:setValue(TC3001, Output ),
				NewButtonFlag = false;
			false -> NewButtonFlag = false
			end,
			NewState = {Frame, Panel, ST2001, TC3001,SelectedCards, HiddenCard, ETS, NewButtonFlag},
			loop(NewState);
			
        #wx{id = CB_ID, event=#wxCommand{type = command_checkbox_clicked, commandInt = Int}} ->
		case ButtonFlag of 
		true ->
			case Int of
			   0 -> 	NewSelectedCards = SelectedCards -- [CB_ID];
			   1 -> 	NewSelectedCards = SelectedCards ++ [CB_ID];
			   true ->  NewSelectedCards = SelectedCards		
			end,
%			io:format("Checkbox id: ~p ~n",[CB_ID]),
%			io:format("NewSelected: ~p ~n", [NewSelectedCards]), 
			_Cards = lists:flatten([ets:match(ets_table,{C,'_','_','_','$1'}) ++ " " || C <- NewSelectedCards]),
%			io:format("ETS table: ~p ~n", [Cards]), 
			wxStaticText:setLabel(ST2001, integer_to_list(length(NewSelectedCards))++ " cards have been selected" ),			
			NewState = {Frame, Panel, ST2001, TC3001,NewSelectedCards, HiddenCard, ETS, ButtonFlag};
		false -> NewState = State
		end,
			loop(NewState);

        #wx{id = _RB_ID, event=#wxCommand{type = command_radiobox_selected, cmdString = Item}} ->
		    case ButtonFlag of true ->
			Stripped_Item = string:strip(Item),
%			io:format("RadioBox id: ~p ~n",[RB_ID]),
%			io:format("Hidden card name: ~p ~n",[Stripped_Item]),
			[[NewHiddenCard]] = ets:match(ets_table, {'$1','_','_','_',Stripped_Item}),
%			io:format("All cards: ~p ~n", [SelectedCards]),
%			io:format("NewHiddenCard: ~p ~n",[NewHiddenCard]),

            %enable button to process based on hidden card 
%			wxButton:new(Panel, 103, [{label, "&Click here to finish"}, {pos,{300,325}}]);

			%clear the TextCtrl
			wxTextCtrl:setValue(TC3001, "" );			
			
			false -> NewHiddenCard = HiddenCard
			end,
			NewState = {Frame, Panel, ST2001, TC3001, SelectedCards, NewHiddenCard, ETS, ButtonFlag},

			loop(NewState);			
			
			
        _Msg ->
            %everything else ends up here
%            io:format("loop default triggered: Got ~n ~p ~n", [Msg]),
            loop(State)

    end.

% each card has a record As = Ace of Spades
% ID number, rank, suit (4 = Spades, 3= Hearts, 2= Diamonds, 1 =Clubs), pip (1-13, 1=Ace 13=King)	
% order is Aces are high, then King, Queen, etc to 2
% rank combines suit and order, thusly As (is 52, highest), Ah (51), Ad, Ac, Ks, Kh, Kd, Kc, etc 2c is lowest
make_ets() ->	
	ETS = ets:new('ets_table', [named_table]),	
	ets:insert(ets_table, [
	{6401,52,4,1, "As"}, {6402,4,4,2,"2s"}, {6403,8,4,3,"3s"},{6404,12,4,4,"4s"}, {6405,16,4,5,"5s"}, {6406,20,4,6,"6s"},{6407,24,4,7,"7s"}, {6408,28,4,8,"8s"}, {6409,32,4,9,"9s"}, {6410,36,4,10,"Ts"}, {6411,40,4,11,"Js"}, {6412,44,4,12,"Qs"}, {6413,48,4,13,"Ks"},
	
	{6301,51,3,1, "Ah"}, {6302,3,3,2,"2h"}, {6303,7,3,3,"3h"},{6304,11,3,4,"4h"}, {6305,15,3,5,"5h"}, {6306,19,3,6,"6h"},{6307,23,3,7,"7h"}, {6308,27,3,8,"8h"}, {6309,31,3,9,"9h"}, {6310,35,3,10,"Th"},{ 6311,39,3,11,"Jh"}, {6312,43,3,12,"Qh"}, {6313,47,3,13,"Kh" },
	
	{6201,50,2,1, "Ad"}, {6202,2,2,2,"2d"}, {6203,6,2,3,"3d"},{6204,10,2,4,"4d"}, {6205,14,2,5,"5d"}, {6206,18,2,6,"6d"},{6207,22,2,7,"7d"}, {6208,26,2,8,"8d"}, {6209,30,2,9,"9d"}, {6210,34,2,10,"Td"}, {6211,38,2,11,"Jd"}, {6212,42,2,12,"Qd"}, {6213,46,2,13,"Kd" },
	
	{6101,49,1,1, "Ac"}, {6102,1,1,2,"2c"}, {6103, 5,1,3,"3c"},{6104,9,1,4,"4c"}, {6105,13,1,5,"5c"}, {6106,17,1,6,"6c"},{6107,21,1,7,"7c"}, {6108,25,1,8,"8c"}, {6109,29,1,9,"9c"}, {6110,33,1,10,"Tc"}, {6111,37,1,11,"Jc"}, {6112,41,1,12,"Qc"}, {6113,45,1,13,"Kc" }]
	),
	ETS.
        	
